
/*@ predicate sorted(int t[],int i,int j) {
  @   \forall int k; i <= k < j => t[k] <= t[k+1]
  @ }
  @*/


/*@ requires 0 <= p <= r && p <= i <= r && p <= j <= r && \valid_range(t,p,r)  
  @ assigns t[i],t[j]
  @ ensures t[i] == \old(t[j]) && t[j] == \old(t[i]) &&
  @       (\forall int k; p<=k<=r => 
  @           (\exists int l; p<=l<=r && t[l] == \old(t[k]))) &&
  @       (\forall int k; p<=k<=r => 
  @           (\exists int l; p<=l<=r && \old(t[l]) == t[k]))
  @*/
void swap(int t[], int i, int j, int p, int r) {
  int tmp = t[i];
  t[i] = t[j];
  t[j] = tmp;
}


/*@ requires 0 <= p <= r && \valid_range(A,p,r)  
  @ ensures  
  @      p <= \result <= r && 
  @      (\forall int l; p <= l < \result => A[l] <= A[\result]) && 
  @      (\forall int l; \result < l <= r => A[l] >  A[\result]) && 
  @      A[\result] == \old(A[r]) &&
  @      (\forall int k; p<=k<=r => 
  @         (\exists int l; p<=l<=r && A[l] == \old(A[k]))) &&
  @      (\forall int k; p<=k<=r => 
  @         (\exists int l; p<=l<=r && \old(A[l]) == A[k])) &&
  @      (\forall int k; \valid_index(A,k) => (k<p || k>r) 
  @                      => \old(A[k]) == A[k])
  @*/
int partition (int A[], int p, int r) 
{ 
  int x = A[r]; 
  int tmp, j, i = p-1; 
  /*@ invariant 
    @    p <= j <= r && p-1 <= i < j && 
    @    (\forall int k; (p <= k <= i) => A[k] <= x) &&
    @    (\forall int k; (i <  k <  j) => A[k] >  x) &&
    @    A[r] == x &&
    @    (\forall int k; p<=k<=r => 
    @       (\exists int l; p<=l<=r && A[l] == \old(A[k]))) &&
    @    (\forall int k; p<=k<=r => 
    @       (\exists int l; p<=l<=r && \old(A[l]) == A[k])) &&
    @    (\forall int k; \valid_index(A,k) => (k<p || k>r) 
    @                    => \old(A[k]) == A[k])  
    @ variant (r-j)
    @*/
  for(j=p; j<r; j++) 
    if (A[j] <= x) { 
      i++;  
      swap(A,i,j,p,r);
    } 
  swap(A,i+1,r,p,r);
  return i+1; 
}




/*@ requires 0 <= p <= r  && \valid_range(A,p,r) 
  @*/
void quicksort(int A[], int p, int r) 
{ 
  int q;
  q = partition(A,p,r);
  //@ assert (q-1)-p < r-p
  if (p < q-1) quicksort(A,p,q-1); 
  //@ assert r-(q+1) < r-p
  if (q+1 < r) quicksort(A,q+1,r); 
} 


